import { Directive, forwardRef, ElementRef, Renderer2, Input, NgModule, Component, ApplicationRef, ComponentFactoryResolver } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { FloatingMenuPlugin, FloatingMenuPluginKey } from '@tiptap/extension-floating-menu';
import { BubbleMenuPlugin, BubbleMenuPluginKey } from '@tiptap/extension-bubble-menu';
import { NodeView } from '@tiptap/core';

class EditorDirective {
    constructor(el, _renderer) {
        this.el = el;
        this._renderer = _renderer;
        this.outputFormat = 'html';
        this.onChange = () => { };
        this.onTouched = () => { };
        this.handleChange = ({ transaction }) => {
            if (!transaction.docChanged) {
                return;
            }
            if (this.outputFormat === 'html') {
                this.onChange(this.editor.getHTML());
                return;
            }
            this.onChange(this.editor.getJSON());
        };
    }
    // Writes a new value to the element.
    // This methods is called when programmatic changes from model to view are requested.
    writeValue(value) {
        if (!value) {
            return;
        }
        if (!this.outputFormat && typeof value === 'string') {
            this.outputFormat = 'html';
        }
        this.editor.chain().setContent(value, false).run();
    }
    // Registers a callback function that is called when the control's value changes in the UI.
    registerOnChange(fn) {
        this.onChange = fn;
    }
    // Registers a callback function that is called by the forms API on initialization to update the form model on blur.
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    // Called by the forms api to enable or disable the element
    setDisabledState(isDisabled) {
        this.editor.setEditable(!isDisabled);
        this._renderer.setProperty(this.el.nativeElement, 'disabled', isDisabled);
    }
    ngOnInit() {
        if (!this.editor) {
            throw new Error('Required: Input `editor`');
        }
        // take the inner contents and clear the block
        const innerHTML = this.el.nativeElement.innerHTML;
        this.el.nativeElement.innerHTML = '';
        // insert the editor in the dom
        this.el.nativeElement.appendChild(this.editor.options.element.firstChild);
        // update the options for the editor
        this.editor.setOptions({ element: this.el.nativeElement });
        // update content to the editor
        if (innerHTML) {
            this.editor.chain().setContent(innerHTML, false).run();
        }
        // register blur handler to update `touched` property
        this.editor.on('blur', () => {
            this.onTouched();
        });
        // register transaction handler to emit changes on update
        this.editor.on('transaction', this.handleChange);
    }
    ngOnDestroy() {
        this.editor.destroy();
    }
}
EditorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'tiptap[editor], [tiptap][editor], tiptap-editor[editor], [tiptapEditor][editor]',
                providers: [{
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => EditorDirective),
                        multi: true
                    }]
            },] }
];
EditorDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
EditorDirective.propDecorators = {
    editor: [{ type: Input }],
    outputFormat: [{ type: Input }]
};

class FloatingMenuDirective {
    constructor(_el) {
        this._el = _el;
        this.tippyOptions = {};
    }
    ngOnInit() {
        if (!this.editor) {
            throw new Error('Required: Input `editor`');
        }
        this.editor.registerPlugin(FloatingMenuPlugin({
            editor: this.editor,
            element: this._el.nativeElement,
            tippyOptions: this.tippyOptions
        }));
    }
    ngOnDestroy() {
        this.editor.unregisterPlugin(FloatingMenuPluginKey);
    }
}
FloatingMenuDirective.decorators = [
    { type: Directive, args: [{
                selector: 'tiptap-floting-menu[editor], [tiptapFloatingMenu][editor]'
            },] }
];
FloatingMenuDirective.ctorParameters = () => [
    { type: ElementRef }
];
FloatingMenuDirective.propDecorators = {
    editor: [{ type: Input }],
    tippyOptions: [{ type: Input }]
};

class BubbleMenuDirective {
    constructor(_el) {
        this._el = _el;
        this.tippyOptions = {};
    }
    ngOnInit() {
        if (!this.editor) {
            throw new Error('Required: Input `editor`');
        }
        this.editor.registerPlugin(BubbleMenuPlugin({
            editor: this.editor,
            element: this._el.nativeElement,
            tippyOptions: this.tippyOptions
        }));
    }
    ngOnDestroy() {
        this.editor.unregisterPlugin(BubbleMenuPluginKey);
    }
}
BubbleMenuDirective.decorators = [
    { type: Directive, args: [{
                selector: 'tiptap-bubble-menu[editor], [tiptapBubbleMenu][editor]'
            },] }
];
BubbleMenuDirective.ctorParameters = () => [
    { type: ElementRef }
];
BubbleMenuDirective.propDecorators = {
    editor: [{ type: Input }],
    tippyOptions: [{ type: Input }]
};

class NgxTiptapModule {
}
NgxTiptapModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    EditorDirective,
                    FloatingMenuDirective,
                    BubbleMenuDirective,
                ],
                exports: [
                    EditorDirective,
                    FloatingMenuDirective,
                    BubbleMenuDirective
                ]
            },] }
];

class AngularNodeViewComponent {
}
AngularNodeViewComponent.decorators = [
    { type: Component, args: [{ template: '' },] }
];
AngularNodeViewComponent.propDecorators = {
    props: [{ type: Input }]
};
class AngularNodeView extends NodeView {
    constructor() {
        super(...arguments);
        this.updateProps = (props) => {
            this.componentRef.instance.props = Object.assign(Object.assign({}, this.componentRef.instance.props), props);
        };
    }
    mount() {
        const injector = this.options.injector;
        this.applicationRef = injector.get(ApplicationRef);
        const componentFactoryResolver = injector.get(ComponentFactoryResolver);
        const factory = componentFactoryResolver.resolveComponentFactory(this.component);
        this.componentRef = factory.create(injector, []);
        // Attach to the view so that the change detector knows to run
        this.applicationRef.attachView(this.componentRef.hostView);
        const props = {
            editor: this.editor,
            node: this.node,
            decorations: this.decorations,
            selected: false,
            extension: this.extension,
            getPos: () => this.getPos(),
            updateAttributes: (attributes = {}) => this.updateAttributes(attributes),
        };
        // Pass input props to the component
        this.componentRef.instance.props = props;
        this.renderer = this.componentRef.injector.get(ElementRef);
        this.contentDOMElement = this.node.isLeaf ? null : document.createElement(this.node.isInline ? 'span' : 'div');
        if (this.contentDOMElement) {
            // For some reason the whiteSpace prop is not inherited properly in Chrome and Safari
            // With this fix it seems to work fine
            // See: https://github.com/ueberdosis/tiptap/issues/1197
            // this.contentDOMElement.style.whiteSpace = 'inherit'
        }
    }
    get dom() {
        return this.renderer.nativeElement;
    }
    get contentDOM() {
        if (this.node.isLeaf) {
            return null;
        }
        const contentElement = this.dom.querySelector('[data-node-view-content]');
        if (this.contentDOMElement
            && contentElement
            && !contentElement.contains(this.contentDOMElement)) {
            contentElement.appendChild(this.contentDOMElement);
        }
        return this.contentDOMElement;
    }
    update(node, decorations) {
        if (this.options.update) {
            return this.options.update(node, decorations);
        }
        if (node.type !== this.node.type) {
            return false;
        }
        if (node === this.node && this.decorations === decorations) {
            return true;
        }
        this.node = node;
        this.decorations = decorations;
        this.updateProps({ node, decorations });
        return true;
    }
    stopEvent(event) {
        var _a, _b, _c;
        return (_c = (_b = (_a = this.options).stopEvent) === null || _b === void 0 ? void 0 : _b.call(_a, event)) !== null && _c !== void 0 ? _c : null;
    }
    selectNode() {
        this.updateProps({ selected: true });
    }
    deselectNode() {
        this.updateProps({ selected: false });
    }
    destroy() {
        this.applicationRef.detachView(this.componentRef.hostView);
    }
}
const AngularNodeViewRenderer = (component, options) => {
    return (props) => {
        return new AngularNodeView(component, props, options);
    };
};

/*
 * Public API Surface of ngx-tiptap
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AngularNodeViewComponent, AngularNodeViewRenderer, BubbleMenuDirective, EditorDirective, FloatingMenuDirective, NgxTiptapModule };
//# sourceMappingURL=ngx-tiptap.js.map
